@@ -17,6 +17,7 @@
 
 #include "arrow/compute/row/grouper.h"
 
+#include <iostream>
 #include <memory>
 #include <mutex>
 #include <type_traits>
@@ -25,12 +26,12 @@
 
 #include "arrow/compute/api_vector.h"
 #include "arrow/compute/function.h"
-#include "arrow/compute/kernels/row_encoder_internal.h"
-#include "arrow/compute/key_hash.h"
-#include "arrow/compute/light_array.h"
+#include "arrow/compute/key_hash_internal.h"
+#include "arrow/compute/light_array_internal.h"
 #include "arrow/compute/registry.h"
 #include "arrow/compute/row/compare_internal.h"
 #include "arrow/compute/row/grouper_internal.h"
+#include "arrow/compute/row/row_encoder_internal.h"
 #include "arrow/type.h"
 #include "arrow/type_traits.h"
 #include "arrow/util/bitmap_ops.h"
@@ -54,18 +55,12 @@ using group_id_t = std::remove_const<decltype(kNoGroupId)>::type;
 using GroupIdType = CTypeTraits<group_id_t>::ArrowType;
 auto g_group_id_type = std::make_shared<GroupIdType>();
 
-inline const uint8_t* GetValuesAsBytes(const ArraySpan& data, int64_t offset = 0) {
-  DCHECK_GT(data.type->byte_width(), 0);
-  int64_t absolute_byte_offset = (data.offset + offset) * data.type->byte_width();
-  return data.GetValues<uint8_t>(1, absolute_byte_offset);
-}
-
 template <typename Value>
+ARROW_DEPRECATED("Deprecated in 18.0.0 along with GetSegments.")
 Status CheckForGetNextSegment(const std::vector<Value>& values, int64_t length,
                               int64_t offset, const std::vector<TypeHolder>& key_types) {
-  if (offset < 0 || offset > length) {
-    return Status::Invalid("invalid grouping segmenter offset: ", offset);
-  }
+  DCHECK_GE(offset, 0);
+  DCHECK_LE(offset, length);
   if (values.size() != key_types.size()) {
     return Status::Invalid("expected batch size ", key_types.size(), " but got ",
                            values.size());
@@ -82,11 +77,22 @@ Status CheckForGetNextSegment(const std::vector<Value>& values, int64_t length,
 }
 
 template <typename Batch>
+ARROW_DEPRECATED("Deprecated in 18.0.0 along with GetSegments.")
 enable_if_t<std::is_same<Batch, ExecSpan>::value || std::is_same<Batch, ExecBatch>::value,
-            Status>
-CheckForGetNextSegment(const Batch& batch, int64_t offset,
-                       const std::vector<TypeHolder>& key_types) {
+            Status> CheckForGetNextSegment(const Batch& batch, int64_t offset,
+                                           const std::vector<TypeHolder>& key_types) {
+  ARROW_SUPPRESS_DEPRECATION_WARNING
   return CheckForGetNextSegment(batch.values, batch.length, offset, key_types);
+  ARROW_UNSUPPRESS_DEPRECATION_WARNING
+}
+
+Status CheckForGetSegments(const ExecSpan& batch,
+                           const std::vector<TypeHolder>& key_types) {
+  // TODO: Move the implementation of CheckForGetNextSegment here once we remove the
+  // deprecated functions.
+  ARROW_SUPPRESS_DEPRECATION_WARNING
+  return CheckForGetNextSegment(batch, 0, key_types);
+  ARROW_UNSUPPRESS_DEPRECATION_WARNING
 }
 
 struct BaseRowSegmenter : public RowSegmenter {
@@ -102,21 +108,6 @@ Segment MakeSegment(int64_t batch_length, int64_t offset, int64_t length, bool e
   return Segment{offset, length, offset + length >= batch_length, extends};
 }
 
-// Used by SimpleKeySegmenter::GetNextSegment to find the match-length of a value within a
-// fixed-width buffer
-int64_t GetMatchLength(const uint8_t* match_bytes, int64_t match_width,
-                       const uint8_t* array_bytes, int64_t offset, int64_t length) {
-  int64_t cursor, byte_cursor;
-  for (cursor = offset, byte_cursor = match_width * cursor; cursor < length;
-       cursor++, byte_cursor += match_width) {
-    if (memcmp(match_bytes, array_bytes + byte_cursor,
-               static_cast<size_t>(match_width)) != 0) {
-      break;
-    }
-  }
-  return std::min(cursor, length) - offset;
-}
-
 using ExtendFunc = std::function<bool(const void*)>;
 constexpr bool kDefaultExtends = true;  // by default, the first segment extends
 constexpr bool kEmptyExtends = true;    // an empty segment extends too
@@ -130,9 +121,22 @@ struct NoKeysSegmenter : public BaseRowSegmenter {
 
   Status Reset() override { return Status::OK(); }
 
+  ARROW_DEPRECATED("Deprecated in 18.0.0. Use GetSegments instead.")
   Result<Segment> GetNextSegment(const ExecSpan& batch, int64_t offset) override {
+    ARROW_SUPPRESS_DEPRECATION_WARNING
     ARROW_RETURN_NOT_OK(CheckForGetNextSegment(batch, offset, {}));
     return MakeSegment(batch.length, offset, batch.length - offset, kDefaultExtends);
+    ARROW_UNSUPPRESS_DEPRECATION_WARNING
+  }
+
+  Result<std::vector<Segment>> GetSegments(const ExecSpan& batch) override {
+    RETURN_NOT_OK(CheckForGetSegments(batch, {}));
+
+    if (batch.length == 0) {
+      return std::vector<Segment>{};
+    }
+    return std::vector<Segment>{
+        MakeSegment(batch.length, 0, batch.length - 0, kDefaultExtends)};
   }
 };
 
@@ -147,13 +151,6 @@ struct SimpleKeySegmenter : public BaseRowSegmenter {
         save_key_data_(static_cast<size_t>(key_type_.type->byte_width())),
         extend_was_called_(false) {}
 
-  Status CheckType(const DataType& type) {
-    if (!is_fixed_width(type)) {
-      return Status::Invalid("SimpleKeySegmenter does not support type ", type);
-    }
-    return Status::OK();
-  }
-
   Status Reset() override {
     extend_was_called_ = false;
     return Status::OK();
@@ -161,7 +158,8 @@ struct SimpleKeySegmenter : public BaseRowSegmenter {
 
   // Checks whether the given grouping data extends the current segment, i.e., is equal to
   // previously seen grouping data, which is updated with each invocation.
-  bool Extend(const void* data) {
+  ARROW_DEPRECATED("Deprecated in 18.0.0 along with GetSegments.")
+  bool ExtendDeprecated(const void* data) {
     bool extends = !extend_was_called_
                        ? kDefaultExtends
                        : 0 == memcmp(save_key_data_.data(), data, save_key_data_.size());
@@ -170,42 +168,125 @@ struct SimpleKeySegmenter : public BaseRowSegmenter {
     return extends;
   }
 
-  Result<Segment> GetNextSegment(const Scalar& scalar, int64_t offset, int64_t length) {
-    ARROW_RETURN_NOT_OK(CheckType(*scalar.type));
-    if (!scalar.is_valid) {
-      return Status::Invalid("segmenting an invalid scalar");
-    }
+  ARROW_DEPRECATED("Deprecated in 18.0.0 along with GetSegments.")
+  Result<Segment> GetNextSegmentDeprecated(const Scalar& scalar, int64_t offset,
+                                           int64_t length) {
+    ARROW_SUPPRESS_DEPRECATION_WARNING
+    DCHECK(is_fixed_width(*scalar.type));
+    DCHECK(scalar.is_valid);
     auto data = checked_cast<const PrimitiveScalarBase&>(scalar).data();
-    bool extends = length > 0 ? Extend(data) : kEmptyExtends;
+    bool extends = length > 0 ? ExtendDeprecated(data) : kEmptyExtends;
     return MakeSegment(length, offset, length, extends);
+    ARROW_UNSUPPRESS_DEPRECATION_WARNING
   }
 
-  Result<Segment> GetNextSegment(const DataType& array_type, const uint8_t* array_bytes,
-                                 int64_t offset, int64_t length) {
-    RETURN_NOT_OK(CheckType(array_type));
+  ARROW_DEPRECATED("Deprecated in 18.0.0 along with GetSegments.")
+  Result<Segment> GetNextSegmentDeprecated(const DataType& array_type,
+                                           const uint8_t* array_bytes, int64_t offset,
+                                           int64_t length) {
+    ARROW_SUPPRESS_DEPRECATION_WARNING
+    DCHECK(is_fixed_width(array_type));
     DCHECK_LE(offset, length);
     int64_t byte_width = array_type.byte_width();
     int64_t match_length = GetMatchLength(array_bytes + offset * byte_width, byte_width,
                                           array_bytes, offset, length);
-    bool extends = length > 0 ? Extend(array_bytes + offset * byte_width) : kEmptyExtends;
+    bool extends =
+        length > 0 ? ExtendDeprecated(array_bytes + offset * byte_width) : kEmptyExtends;
     return MakeSegment(length, offset, match_length, extends);
+    ARROW_UNSUPPRESS_DEPRECATION_WARNING
   }
 
   Result<Segment> GetNextSegment(const ExecSpan& batch, int64_t offset) override {
+    ARROW_SUPPRESS_DEPRECATION_WARNING
     ARROW_RETURN_NOT_OK(CheckForGetNextSegment(batch, offset, {key_type_}));
     if (offset == batch.length) {
       return MakeSegment(batch.length, offset, 0, kEmptyExtends);
     }
     const auto& value = batch.values[0];
     if (value.is_scalar()) {
-      return GetNextSegment(*value.scalar, offset, batch.length);
+      return GetNextSegmentDeprecated(*value.scalar, offset, batch.length);
     }
     ARROW_DCHECK(value.is_array());
     const auto& array = value.array;
-    if (array.GetNullCount() > 0) {
-      return Status::NotImplemented("segmenting a nullable array");
+    DCHECK_EQ(array.GetNullCount(), 0);
+    return GetNextSegmentDeprecated(*array.type, GetValuesAsBytes(array), offset,
+                                    batch.length);
+    ARROW_UNSUPPRESS_DEPRECATION_WARNING
+  }
+
+  Result<std::vector<Segment>> GetSegments(const ExecSpan& batch) override {
+    RETURN_NOT_OK(CheckForGetSegments(batch, {key_type_}));
+
+    if (batch.length == 0) {
+      return std::vector<Segment>{};
     }
-    return GetNextSegment(*array.type, GetValuesAsBytes(array), offset, batch.length);
+
+    const auto& value = batch.values[0];
+    DCHECK(is_fixed_width(*value.type()));
+
+    std::vector<Segment> segments;
+    const void* key_data;
+    if (value.is_scalar()) {
+      const auto& scalar = *value.scalar;
+      DCHECK(scalar.is_valid);
+      key_data = checked_cast<const PrimitiveScalarBase&>(scalar).data();
+      bool extends = Extend(key_data);
+      segments.push_back(MakeSegment(batch.length, 0, batch.length, extends));
+    } else {
+      DCHECK(value.is_array());
+      const auto& array = value.array;
+      DCHECK_EQ(array.GetNullCount(), 0);
+      auto data = GetValuesAsBytes(array);
+      int64_t byte_width = array.type->byte_width();
+      int64_t offset = 0;
+      bool extends = Extend(data);
+      while (offset < array.length) {
+        int64_t match_length = GetMatchLength(data + offset * byte_width, byte_width,
+                                              data, offset, array.length);
+        segments.push_back(MakeSegment(array.length, offset, match_length,
+                                       offset == 0 ? extends : false));
+        offset += match_length;
+      }
+      key_data = data + (array.length - 1) * byte_width;
+    }
+
+    SaveKeyData(key_data);
+
+    return segments;
+  }
+
+ private:
+  static const uint8_t* GetValuesAsBytes(const ArraySpan& data, int64_t offset = 0) {
+    DCHECK_GT(data.type->byte_width(), 0);
+    int64_t absolute_byte_offset = (data.offset + offset) * data.type->byte_width();
+    return data.GetValues<uint8_t>(1, absolute_byte_offset);
+  }
+
+  // Find the match-length of a value within a fixed-width buffer
+  static int64_t GetMatchLength(const uint8_t* match_bytes, int64_t match_width,
+                                const uint8_t* array_bytes, int64_t offset,
+                                int64_t length) {
+    int64_t cursor, byte_cursor;
+    for (cursor = offset, byte_cursor = match_width * cursor; cursor < length;
+         cursor++, byte_cursor += match_width) {
+      if (memcmp(match_bytes, array_bytes + byte_cursor,
+                 static_cast<size_t>(match_width)) != 0) {
+        break;
+      }
+    }
+    return std::min(cursor, length) - offset;
+  }
+
+  bool Extend(const void* data) {
+    if (ARROW_PREDICT_FALSE(!extend_was_called_)) {
+      extend_was_called_ = true;
+      return kDefaultExtends;
+    }
+    return 0 == memcmp(save_key_data_.data(), data, save_key_data_.size());
+  }
+
+  void SaveKeyData(const void* data) {
+    memcpy(save_key_data_.data(), data, save_key_data_.size());
   }
 
  private:
@@ -217,22 +298,23 @@ struct SimpleKeySegmenter : public BaseRowSegmenter {
 struct AnyKeysSegmenter : public BaseRowSegmenter {
   static Result<std::unique_ptr<RowSegmenter>> Make(
       const std::vector<TypeHolder>& key_types, ExecContext* ctx) {
-    ARROW_RETURN_NOT_OK(Grouper::Make(key_types, ctx));  // check types
-    return std::make_unique<AnyKeysSegmenter>(key_types, ctx);
+    ARROW_ASSIGN_OR_RAISE(auto grouper, Grouper::Make(key_types, ctx));  // check types
+    return std::make_unique<AnyKeysSegmenter>(key_types, ctx, std::move(grouper));
   }
 
-  AnyKeysSegmenter(const std::vector<TypeHolder>& key_types, ExecContext* ctx)
+  AnyKeysSegmenter(const std::vector<TypeHolder>& key_types, ExecContext* ctx,
+                   std::unique_ptr<Grouper> grouper)
       : BaseRowSegmenter(key_types),
-        ctx_(ctx),
-        grouper_(nullptr),
+        grouper_(std::move(grouper)),
         save_group_id_(kNoGroupId) {}
 
   Status Reset() override {
-    grouper_ = nullptr;
+    ARROW_RETURN_NOT_OK(grouper_->Reset());
     save_group_id_ = kNoGroupId;
     return Status::OK();
   }
 
+  ARROW_DEPRECATED("Deprecated in 18.0.0 along with GetSegments.")
   bool Extend(const void* data) {
     auto group_id = *static_cast<const group_id_t*>(data);
     bool extends =
@@ -241,32 +323,13 @@ struct AnyKeysSegmenter : public BaseRowSegmenter {
     return extends;
   }
 
-  // Runs the grouper on a single row.  This is used to determine the group id of the
-  // first row of a new segment to see if it extends the previous segment.
-  template <typename Batch>
-  Result<group_id_t> MapGroupIdAt(const Batch& batch, int64_t offset) {
-    if (!grouper_) return kNoGroupId;
-    ARROW_ASSIGN_OR_RAISE(auto datum, grouper_->Consume(batch, offset,
-                                                        /*length=*/1));
-    if (!datum.is_array()) {
-      return Status::Invalid("accessing unsupported datum kind ", datum.kind());
-    }
-    const std::shared_ptr<ArrayData>& data = datum.array();
-    ARROW_DCHECK(data->GetNullCount() == 0);
-    DCHECK_EQ(data->type->id(), GroupIdType::type_id);
-    DCHECK_EQ(1, data->length);
-    const group_id_t* values = data->GetValues<group_id_t>(1);
-    return values[0];
-  }
-
+  ARROW_DEPRECATED("Deprecated in 18.0.0. Use GetSegments instead.")
   Result<Segment> GetNextSegment(const ExecSpan& batch, int64_t offset) override {
+    ARROW_SUPPRESS_DEPRECATION_WARNING
     ARROW_RETURN_NOT_OK(CheckForGetNextSegment(batch, offset, key_types_));
     if (offset == batch.length) {
       return MakeSegment(batch.length, offset, 0, kEmptyExtends);
     }
-    // ARROW-18311: make Grouper support Reset()
-    // so it can be reset instead of recreated below
-    //
     // the group id must be computed prior to resetting the grouper, since it is compared
     // to save_group_id_, and after resetting the grouper produces incomparable group ids
     ARROW_ASSIGN_OR_RAISE(auto group_id, MapGroupIdAt(batch, offset));
@@ -276,45 +339,98 @@ struct AnyKeysSegmenter : public BaseRowSegmenter {
       return extends;
     };
     // resetting drops grouper's group-ids, freeing-up memory for the next segment
-    ARROW_ASSIGN_OR_RAISE(grouper_, Grouper::Make(key_types_, ctx_));  // TODO: reset it
-    // GH-34475: cache the grouper-consume result across invocations of GetNextSegment
+    ARROW_RETURN_NOT_OK(grouper_->Reset());
+
     ARROW_ASSIGN_OR_RAISE(auto datum, grouper_->Consume(batch, offset));
-    if (datum.is_array()) {
-      // `data` is an array whose index-0 corresponds to index `offset` of `batch`
-      const std::shared_ptr<ArrayData>& data = datum.array();
-      DCHECK_EQ(data->length, batch.length - offset);
-      ARROW_DCHECK(data->GetNullCount() == 0);
-      DCHECK_EQ(data->type->id(), GroupIdType::type_id);
-      const group_id_t* values = data->GetValues<group_id_t>(1);
-      int64_t cursor;
-      for (cursor = 1; cursor < data->length; cursor++) {
-        if (values[0] != values[cursor]) break;
-      }
-      int64_t length = cursor;
-      bool extends = length > 0 ? bound_extend(values) : kEmptyExtends;
-      return MakeSegment(batch.length, offset, length, extends);
-    } else {
-      return Status::Invalid("segmenting unsupported datum kind ", datum.kind());
+    DCHECK(datum.is_array());
+    // `data` is an array whose index-0 corresponds to index `offset` of `batch`
+    const std::shared_ptr<ArrayData>& data = datum.array();
+    DCHECK_EQ(data->length, batch.length - offset);
+    DCHECK_EQ(data->GetNullCount(), 0);
+    DCHECK_EQ(data->type->id(), GroupIdType::type_id);
+    const group_id_t* values = data->GetValues<group_id_t>(1);
+    int64_t cursor;
+    for (cursor = 1; cursor < data->length; cursor++) {
+      if (values[0] != values[cursor]) break;
     }
+    int64_t length = cursor;
+    bool extends = length > 0 ? bound_extend(values) : kEmptyExtends;
+    return MakeSegment(batch.length, offset, length, extends);
+    ARROW_UNSUPPRESS_DEPRECATION_WARNING
+  }
+
+  Result<std::vector<Segment>> GetSegments(const ExecSpan& batch) override {
+    RETURN_NOT_OK(CheckForGetSegments(batch, {key_types_}));
+
+    if (batch.length == 0) {
+      return std::vector<Segment>{};
+    }
+
+    // determine if the first segment in this batch extends the last segment in the
+    // previous batch
+    bool extends = kDefaultExtends;
+    if (save_group_id_ != kNoGroupId) {
+      // the group id must be computed prior to resetting the grouper, since it is
+      // compared to save_group_id_, and after resetting the grouper produces incomparable
+      // group ids
+      ARROW_ASSIGN_OR_RAISE(auto group_id, MapGroupIdAt(batch));
+      // it "extends" unless the group id differs from the last group id
+      extends = (group_id == save_group_id_);
+    }
+
+    // resetting drops grouper's group-ids, freeing-up memory for the next segment
+    RETURN_NOT_OK(grouper_->Reset());
+
+    std::vector<Segment> segments;
+    ARROW_ASSIGN_OR_RAISE(auto datum, grouper_->Consume(batch));
+    DCHECK(datum.is_array());
+    // `data` is an array whose index-0 corresponds to index `offset` of `batch`
+    const std::shared_ptr<ArrayData>& data = datum.array();
+    DCHECK_EQ(data->length, batch.length);
+    DCHECK_EQ(data->GetNullCount(), 0);
+    DCHECK_EQ(data->type->id(), GroupIdType::type_id);
+    const group_id_t* group_ids = data->GetValues<group_id_t>(1);
+    int64_t current_group_offset = 0;
+    int64_t cursor;
+    for (cursor = 1; cursor < data->length; ++cursor) {
+      if (group_ids[cursor] != group_ids[current_group_offset]) {
+        segments.push_back(MakeSegment(batch.length, current_group_offset,
+                                       cursor - current_group_offset,
+                                       current_group_offset == 0 ? extends : false));
+        current_group_offset = cursor;
+      }
+    }
+    segments.push_back(MakeSegment(batch.length, current_group_offset,
+                                   cursor - current_group_offset,
+                                   current_group_offset == 0 ? extends : false));
+
+    // update the save_group_id_ to the last group id in this batch
+    save_group_id_ = group_ids[batch.length - 1];
+
+    return segments;
+  }
+
+ private:
+  // Runs the grouper on a single row.  This is used to determine the group id of the
+  // first row of a new segment to see if it extends the previous segment.
+  template <typename Batch>
+  Result<group_id_t> MapGroupIdAt(const Batch& batch, int64_t offset = 0) {
+    ARROW_ASSIGN_OR_RAISE(auto datum, grouper_->Consume(batch, offset,
+                                                        /*length=*/1));
+    DCHECK(datum.is_array());
+    const std::shared_ptr<ArrayData>& data = datum.array();
+    DCHECK_EQ(data->GetNullCount(), 0);
+    DCHECK_EQ(data->type->id(), GroupIdType::type_id);
+    DCHECK_EQ(1, data->length);
+    const group_id_t* values = data->GetValues<group_id_t>(1);
+    return values[0];
   }
 
  private:
-  ExecContext* const ctx_;
   std::unique_ptr<Grouper> grouper_;
   group_id_t save_group_id_;
 };
 
-Status CheckAndCapLengthForConsume(int64_t batch_length, int64_t& consume_offset,
-                                   int64_t* consume_length) {
-  if (consume_offset < 0) {
-    return Status::Invalid("invalid grouper consume offset: ", consume_offset);
-  }
-  if (*consume_length < 0) {
-    *consume_length = batch_length - consume_offset;
-  }
-  return Status::OK();
-}
-
 }  // namespace
 
 Result<std::unique_ptr<RowSegmenter>> MakeAnyKeysSegmenter(
@@ -337,36 +453,16 @@ Result<std::unique_ptr<RowSegmenter>> RowSegmenter::Make(
 
 namespace {
 
-struct GrouperNoKeysImpl : Grouper {
-  Result<std::shared_ptr<Array>> MakeConstantGroupIdArray(int64_t length,
-                                                          group_id_t value) {
-    std::unique_ptr<ArrayBuilder> a_builder;
-    RETURN_NOT_OK(MakeBuilder(default_memory_pool(), g_group_id_type, &a_builder));
-    using GroupIdBuilder = typename TypeTraits<GroupIdType>::BuilderType;
-    auto builder = checked_cast<GroupIdBuilder*>(a_builder.get());
-    if (length != 0) {
-      RETURN_NOT_OK(builder->Resize(length));
-    }
-    for (int64_t i = 0; i < length; i++) {
-      builder->UnsafeAppend(value);
-    }
-    std::shared_ptr<Array> array;
-    RETURN_NOT_OK(builder->Finish(&array));
-    return std::move(array);
+Status CheckAndCapLengthForConsume(int64_t batch_length, int64_t& consume_offset,
+                                   int64_t* consume_length) {
+  if (consume_offset < 0) {
+    return Status::Invalid("invalid grouper consume offset: ", consume_offset);
   }
-  Result<Datum> Consume(const ExecSpan& batch, int64_t offset, int64_t length) override {
-    ARROW_ASSIGN_OR_RAISE(auto array, MakeConstantGroupIdArray(length, 0));
-    return Datum(array);
+  if (*consume_length < 0) {
+    *consume_length = batch_length - consume_offset;
   }
-  Result<ExecBatch> GetUniques() override {
-    auto data = ArrayData::Make(uint32(), 1, 0);
-    auto values = data->GetMutableValues<uint32_t>(0);
-    values[0] = 0;
-    ExecBatch out({Datum(data)}, 1);
-    return std::move(out);
-  }
-  uint32_t num_groups() const override { return 1; }
-};
+  return Status::OK();
+}
 
 struct GrouperImpl : public Grouper {
   static Result<std::unique_ptr<GrouperImpl>> Make(
@@ -416,7 +512,15 @@ struct GrouperImpl : public Grouper {
       return Status::NotImplemented("Keys of type ", *key);
     }
 
-    return std::move(impl);
+    return impl;
+  }
+
+  Status Reset() override {
+    map_.clear();
+    offsets_.clear();
+    key_bytes_.clear();
+    num_groups_ = 0;
+    return Status::OK();
   }
 
   Result<Datum> Consume(const ExecSpan& batch, int64_t offset, int64_t length) override {
@@ -546,7 +650,7 @@ struct GrouperFastImpl : public Grouper {
       const TypeHolder& key = keys[icol];
       if (key.id() == Type::DICTIONARY) {
         auto bit_width = checked_cast<const FixedWidthType&>(*key).bit_width();
-        ARROW_DCHECK(bit_width % 8 == 0);
+        DCHECK_EQ(bit_width % 8, 0);
         impl->col_metadata_[icol] = KeyColumnMetadata(true, bit_width / 8);
       } else if (key.id() == Type::BOOL) {
         impl->col_metadata_[icol] = KeyColumnMetadata(true, 0);
@@ -592,10 +696,21 @@ struct GrouperFastImpl : public Grouper {
     impl->minibatch_hashes_.resize(impl->minibatch_size_max_ +
                                    kPaddingForSIMD / sizeof(uint32_t));
 
-    return std::move(impl);
+    return impl;
   }
 
-  ~GrouperFastImpl() { map_.cleanup(); }
+  Status Reset() override {
+    ARROW_DCHECK_EQ(temp_stack_.AllocatedSize(), 0);
+    rows_.Clean();
+    rows_minibatch_.Clean();
+    map_.cleanup();
+    RETURN_NOT_OK(map_.init(encode_ctx_.hardware_flags, ctx_->memory_pool()));
+    // TODO: It is now assumed that the dictionaries_ are identical to the first batch
+    // throughout the grouper's lifespan so no resetting is needed. But if we want to
+    // support different dictionaries for different batches, we need to reset the
+    // dictionaries_ here.
+    return Status::OK();
+  }
 
   Result<Datum> Consume(const ExecSpan& batch, int64_t offset, int64_t length) override {
     ARROW_RETURN_NOT_OK(CheckAndCapLengthForConsume(batch.length, offset, &length));
@@ -762,6 +877,9 @@ struct GrouperFastImpl : public Grouper {
       } else {
         ARROW_ASSIGN_OR_RAISE(fixedlen_bufs[i],
                               AllocatePaddedBuffer((num_groups + 1) * sizeof(uint32_t)));
+        // Set offset[0] to 0 so the later allocation of varlen_bufs doesn't see an
+        // uninitialized value when num_groups == 0.
+        reinterpret_cast<uint32_t*>(fixedlen_bufs[i]->mutable_data())[0] = 0;
       }
       cols_[i] =
           KeyColumnArray(col_metadata_[i], num_groups, non_null_bufs[i]->mutable_data(),
@@ -838,8 +956,7 @@ struct GrouperFastImpl : public Grouper {
     return out;
   }
 
-  static constexpr int log_minibatch_max_ = 10;
-  static constexpr int minibatch_size_max_ = 1 << log_minibatch_max_;
+  static constexpr int minibatch_size_max_ = arrow::util::MiniBatch::kMiniBatchLength;
   static constexpr int minibatch_size_min_ = 128;
   int minibatch_size_;
 
