@@ -26,12 +26,11 @@
 #include "parquet/encryption/file_system_key_material_store.h"
 #include "parquet/encryption/key_toolkit_internal.h"
 
-namespace parquet {
-namespace encryption {
+namespace parquet::encryption {
 
 void CryptoFactory::RegisterKmsClientFactory(
     std::shared_ptr<KmsClientFactory> kms_client_factory) {
-  key_toolkit_.RegisterKmsClientFactory(kms_client_factory);
+  key_toolkit_->RegisterKmsClientFactory(std::move(kms_client_factory));
 }
 
 std::shared_ptr<FileEncryptionProperties> CryptoFactory::GetFileEncryptionProperties(
@@ -59,8 +58,8 @@ std::shared_ptr<FileEncryptionProperties> CryptoFactory::GetFileEncryptionProper
     }
   }
 
-  FileKeyWrapper key_wrapper(&key_toolkit_, kms_connection_config, key_material_store,
-                             encryption_config.cache_lifetime_seconds,
+  FileKeyWrapper key_wrapper(key_toolkit_.get(), kms_connection_config,
+                             key_material_store, encryption_config.cache_lifetime_seconds,
                              encryption_config.double_wrapping);
 
   int32_t dek_length_bits = encryption_config.data_key_length_bits;
@@ -73,8 +72,7 @@ std::shared_ptr<FileEncryptionProperties> CryptoFactory::GetFileEncryptionProper
   int dek_length = dek_length_bits / 8;
 
   std::string footer_key(dek_length, '\0');
-  RandBytes(reinterpret_cast<uint8_t*>(&footer_key[0]),
-            static_cast<int>(footer_key.size()));
+  RandBytes(reinterpret_cast<uint8_t*>(footer_key.data()), footer_key.size());
 
   std::string footer_key_metadata =
       key_wrapper.GetEncryptionKeyMetadata(footer_key, footer_key_id, true);
@@ -149,8 +147,7 @@ ColumnPathToEncryptionPropertiesMap CryptoFactory::GetColumnEncryptionProperties
       }
 
       std::string column_key(dek_length, '\0');
-      RandBytes(reinterpret_cast<uint8_t*>(&column_key[0]),
-                static_cast<int>(column_key.size()));
+      RandBytes(reinterpret_cast<uint8_t*>(column_key.data()), column_key.size());
       std::string column_key_key_metadata =
           key_wrapper->GetEncryptionKeyMetadata(column_key, column_key_id, false);
 
@@ -174,7 +171,7 @@ std::shared_ptr<FileDecryptionProperties> CryptoFactory::GetFileDecryptionProper
     const DecryptionConfiguration& decryption_config, const std::string& file_path,
     const std::shared_ptr<::arrow::fs::FileSystem>& file_system) {
   auto key_retriever = std::make_shared<FileKeyUnwrapper>(
-      &key_toolkit_, kms_connection_config, decryption_config.cache_lifetime_seconds,
+      key_toolkit_, kms_connection_config, decryption_config.cache_lifetime_seconds,
       file_path, file_system);
 
   return FileDecryptionProperties::Builder()
@@ -188,9 +185,8 @@ void CryptoFactory::RotateMasterKeys(
     const std::string& parquet_file_path,
     const std::shared_ptr<::arrow::fs::FileSystem>& file_system, bool double_wrapping,
     double cache_lifetime_seconds) {
-  key_toolkit_.RotateMasterKeys(kms_connection_config, parquet_file_path, file_system,
-                                double_wrapping, cache_lifetime_seconds);
+  key_toolkit_->RotateMasterKeys(kms_connection_config, parquet_file_path, file_system,
+                                 double_wrapping, cache_lifetime_seconds);
 }
 
-}  // namespace encryption
-}  // namespace parquet
+}  // namespace parquet::encryption
